/*
*********************************************************************************************************
*                                               uC/OS-II
*                                         The Real-Time Kernel
*
*                        (c) Copyright 1992-1998, Jean J. Labrosse, Plantation, FL
*                                          All Rights Reserved
*
*
*                                           ARM Specific code
*
* File : OS_CPU_C.C
* By   : N. Douglas
*********************************************************************************************************
*/

#define  OS_CPU_GLOBALS
#include "includes.h"

/*
*********************************************************************************************************
*                                        INITIALIZE A TASK'S STACK
*
* Description: This function is called by either OSTaskCreate() or OSTaskCreateExt() to initialize the
*              stack frame of the task being created.  This function is highly processor specific.
*
* Arguments  : task          is a pointer to the task code
*
*              pdata         is a pointer to a user supplied data area that will be passed to the task
*                            when the task first executes.
*
*              ptos          is a pointer to the top of stack.  It is assumed that 'ptos' points to
*                            a 'free' entry on the task stack.  If OS_STK_GROWTH is set to 1 then 
*                            'ptos' will contain the HIGHEST valid address of the stack.  Similarly, if
*                            OS_STK_GROWTH is set to 0, the 'ptos' will contains the LOWEST valid address
*                            of the stack.
*
*              opt           specifies options that can be used to alter the behavior of OSTaskStkit().
*                            (see uCOS_II.H for OS_TASK_OPT_???).
*
* Returns    : Always returns the location of the new top-of-stack' once the processor registers have
*              been placed on the stack in the proper order.
*********************************************************************************************************
*/

#if OS_TASK_DEL_EN == 1
/* Called should a task ever try returning */
static void SelfDestruct(void)
{
    OSTaskDel(OS_PRIO_SELF);
}
#endif

#if OS_STK_GROWTH == 1
/* Full descending stack */
#define STACKTHIS(stk, item) *--stk = (OS_STK) item
#else
/* Empty ascending stack */
#define STACKTHIS(stk, item) *stk++ = (OS_STK) item
#endif

void *OSTaskStkInit (void (*task)(void *pd), void *pdata, void *ptos, INT16U opt)
{
    OS_STK *stk;

    opt    = opt;                           /* 'opt' is not used, prevent warning                      */
    stk    = (OS_STK *)ptos;                /* Load stack pointer                                      */

    /* Stuff a starting context frame onto the stack */
#if OS_TASK_DEL_EN ==1
    STACKTHIS(stk, SelfDestruct);           /* r14=addr of SelfDestruct routine in case of return      */
#else
    STACKTHIS(stk, 0);                      /* r14=Reset machine in case of return                     */
#endif
    STACKTHIS(stk, ptos);                   /* r13=stack ptr                                           */
    STACKTHIS(stk, 0);                      /* r12=ip                                                  */
    STACKTHIS(stk, 0);                      /* r11=fp                                                  */
    STACKTHIS(stk, ptos);                   /* r10=sl                                                  */
    STACKTHIS(stk, 0);                      /* r9=sb                                                   */
    STACKTHIS(stk, 0);                      /* r8                                                      */
    STACKTHIS(stk, 0);                      /* r7                                                      */
    STACKTHIS(stk, 0);                      /* r6                                                      */
    STACKTHIS(stk, 0);                      /* r5                                                      */
    STACKTHIS(stk, 0);                      /* r4                                                      */
    STACKTHIS(stk, 0);                      /* r3                                                      */
    STACKTHIS(stk, 0);                      /* r2                                                      */
    STACKTHIS(stk, 0);                      /* r1                                                      */
    STACKTHIS(stk, pdata);                  /* r0                                                      */

    STACKTHIS(stk, task);                   /* pc                                                      */
    STACKTHIS(stk, 0x10);                   /* spsr=User32, interrupts enabled, flags flattened        */

    return ((void *)stk);
}

/*$PAGE*/
#if OS_CPU_HOOKS_EN
/*
*********************************************************************************************************
*                                          TASK CREATION HOOK
*
* Description: This function is called when a task is created.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
void OSTaskCreateHook (OS_TCB *ptcb)
{
    ptcb = ptcb;                       /* Prevent compiler warning                                     */
}


/*
*********************************************************************************************************
*                                           TASK DELETION HOOK
*
* Description: This function is called when a task is deleted.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
void OSTaskDelHook (OS_TCB *ptcb)
{
    ptcb = ptcb;                       /* Prevent compiler warning                                     */
}

/*
*********************************************************************************************************
*                                           TASK SWITCH HOOK
*
* Description: This function is called when a task switch is performed.  This allows you to perform other
*              operations during a context switch.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts are disabled during this call.
*              2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                 will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the 
*                 task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/
void OSTaskSwHook (void)
{
	/* Enable the tick ISR */
	extern int SystemStarted;
	SystemStarted=1;
}

/*
*********************************************************************************************************
*                                           STATISTIC TASK HOOK
*
* Description: This function is called every second by uC/OS-II's statistics task.  This allows your 
*              application to add functionality to the statistics task.
*
* Arguments  : none
*********************************************************************************************************
*/
void OSTaskStatHook (void)
{
}

/*
*********************************************************************************************************
*                                               TICK HOOK
*
* Description: This function is called every tick.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/
void OSTimeTickHook (void)
{
}
#endif
